# -*- coding: utf-8 -*-

from django.conf import settings
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.translation import ugettext_lazy as _

from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel, DefaultModel
from jmb.i18n.models import TranslationModel, I18NModel
from jmb.core.db.fields import HTMLField, HTMLFieldTinyMCE

from jmb.address.models import Address

##########################
# MAGAZZINO              #
##########################
class StoreAddress (Address):
    class Meta:
        verbose_name = _("store address")
        verbose_name_plural = _("store addresses")

class StoreManager(models.Manager):
    def default_store(self):
        store = Store.objects.filter(default=True)
        if store:
            store = store[0]
        else:
            try:
                store = Store.objects.all()[0]
            except:
                store = None
        return store

class Store (DateModel, UserModel, StatusModel, DefaultModel):
    address = models.ManyToManyField(
        StoreAddress,
        blank = True, null = True,
        db_index = True,
        verbose_name = _("address"),
    )
    code = models.CharField (
        unique = True,
        max_length = 15,
        db_index = True,
        verbose_name = _("code"),
    )
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("name"),
    )
    description = models.TextField (
        blank = True, null = True,
        verbose_name = _("description"),
    )
    objects = StoreManager()

    class Meta:
        verbose_name = _("store")
        verbose_name_plural = _("stores")
        permissions = (
            ("view_store"    , "Can view store"),
            )

    def __unicode__(self):
        return u"%s" % self.name

################################
#  BOLLETTARIO MAGAZZINO       #
################################
class StoreDocument (DateModel, UserModel, StatusModel):
    code = models.CharField (
        unique = True,
        max_length = 15,
        db_index = True,
        verbose_name = _("code"),
    )
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("name"),
    )
    description = models.TextField (
        blank=True,
        verbose_name = _("description"),
    )
    class Meta:
        verbose_name = _("store document")
        verbose_name_plural = _("store documents")
        permissions = (
            ("view_storedocument"    , "Can view store document"),
            )


    def __unicode__(self):
        return u"%s" % self.name

################################
# NUMERAZIONE BOLLETTARIO      #
################################
class StoreDocumentNumber (DateModel):
    store_document = models.ForeignKey (
        StoreDocument,
        db_index = True,
        verbose_name = _("store document"),
    )
    application = models.CharField (
        max_length = 1,
        choices = settings.STORE_APP_TYPE,
        db_index = True,
    )
    year = models.IntegerField (
        db_index = True,
        verbose_name = _("year"),
    )
    last_date = models.DateField(
        db_index = True,
        verbose_name = _('last date')
    )
    last_number = models.PositiveIntegerField(
        db_index = True,
        verbose_name = _("last number"),
    )

    class Meta:
        verbose_name = _("store document number")
        verbose_name_plural = _("store document numbers")
        unique_together = ("store_document", "year", "application")
        permissions = (
            ("view_storedocumentnumber", "Can view store document number"),
        )

    def __unicode__(self):
        return u"%(name)s - %(app)s - %(year)s: %(num)s - %(ldate)s" % {
            'name':     self.store_document.name,
            'app':      self.application,
            'year':     self.year,
            'num':      self.last_number,
            'ldate':    self.last_date
        }

################################
# CAUSALE MAGAZZINO            #
################################
class StoreCauseManager(models.Manager):
    def get_default(self):
        return self.get_query_set().get(default=True)

class StoreCause (DateModel, UserModel, StatusModel):
    objects = StoreCauseManager()

    store = models.ForeignKey (
        Store,
        db_index = True,
        verbose_name = _("store"),
    )
    store_document = models.ForeignKey(
        StoreDocument,
        null = True, blank = True,
        db_index = True,
        verbose_name = _("store document"),
    )
    code = models.CharField (
        unique = True,
        max_length = 15,
        db_index = True,
        verbose_name = _("code"),
    )
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("name"),
    )
    description = models.TextField (
        blank = True, null = True,
        verbose_name = _("description"),
    )
    category = models.CharField (
        max_length = 3,
        choices = settings.STORE_ORGANIZATIONTYPE_DEF,
        db_index = True,
        verbose_name = _("category"),
    )
    move = models.CharField (
        max_length = 10,
        choices = settings.STORE_MOVE_TYPE,
        db_index = True,
        verbose_name = _("move"),
    )
    ddt = models.BooleanField (
        db_index = True,
        verbose_name = _("ddt required"),
    )
    invoice = models.BooleanField (
        db_index = True,
        verbose_name = _("ivoice required"),
    )
    stock = models.IntegerField (
        max_length = 3,
        choices = settings.STORE_STOCK_BUSY_TYPE,
        db_index = True,
        verbose_name = _("stock"),
    )
    busy = models.IntegerField (
        max_length = 3,
        choices=settings.STORE_STOCK_BUSY_TYPE,
        db_index = True,
        verbose_name = _("busy"),
    )
    default = models.BooleanField(
        default=False,
        verbose_name=_('Default'),
        help_text=_('Is the default store cause?'),
    )
    default_for_shop = models.BooleanField(
        default=False,
        verbose_name=_('Default For Shop'),
        help_text=_("""
            If True all shop orders take this store cause by default.
            Previous store cause set "Default For Shop" will change to False.
        """),
    )
    model_template = models.CharField (
        max_length = 30,
        blank = True, null = True,
        db_index = True,
        verbose_name = _("model template"),
    )

    class Meta:
        verbose_name = _("store cause")
        verbose_name_plural = _("store causes")
        permissions = (
            ("view_storecause", "Can view storecause"),
        )


    def __unicode__(self):
        return u"%s" % self.name

    def save(self, force_insert=False, force_update=False, **kw):
        super(StoreCause, self).save(
            force_insert=force_insert, force_update=force_update, **kw)
        ## se sto creaundo un default
        ## verifico che non esista un altro dello stesso 'store' e della stessa 'category'
        if self.default:
            for sc in StoreCause.objects.exclude(pk=self.id).filter(store=self.store, category=self.category):
                sc.default = False
                sc.save(force_insert=force_insert, 
                        force_update=force_update, **kw)
            super(StoreCause, self).save(
                force_insert=force_insert, force_update=force_update, **kw)

        ## se ho flaggato default_for_shop controllo che non ci sia qualca altra store cause gia
        ## settata come default.
        if self.default_for_shop:
            for sc in StoreCause.objects.exclude(pk=self.id).filter(default_for_shop=True):
                sc.default_for_shop = False
                sc.save(force_insert=force_insert, force_update=force_update, **kw)
            super(StoreCause, self).save(force_insert=force_insert, 
                                         force_update=force_update, **kw)
